# encoding: utf-8
"""
Colorer does enable colored logging messages by using `ANSI escape sequences <http://en.wikipedia.org/wiki/ANSI_escape_code>`_.
The colored output is generated only when the console is a terminal supporting it, so if you redirect the output to a log file
you will not see the escape codes in the file.

>>> import colorer, logging
... logging.error("red line")
... logging.warn("yellow line")
... logging.info("gray line")
... logging.debug("magenta line")
"""
import copy, logging, sys

PY3 = (sys.version_info[0] == 3)
if PY3:
    text_type = str
    binary_type = bytes
else:
    text_type = unicode
    binary_type = str

def ensure_text(s, encoding='utf-8', errors='strict'):
    if isinstance(s, binary_type):
        return s.decode(encoding, errors)
    elif isinstance(s, text_type):
        return s
    else:
        raise TypeError("not expecting type '%s'" % type(s))

def add_coloring_to_emit_ansi(fn):
    # add methods we need to the class
    def new(*args):
        # new_args = args
        new_args = (args[0], copy.copy(args[1])) if len(args) == 2 else (args[0], copy.copy(args[1]), args[2:])
        if hasattr(args[0], 'baseFilename'):
            return fn(*args)
        levelno = new_args[1].levelno
        if levelno >= 50:
            color = '\x1b[31m'  # CRITICAL red
        elif levelno >= 40:
            color = '\x1b[31m'  # ERROR ligth red
        elif levelno >= 30:
            color = '\x1b[33m'  # WARNING light yellow
        elif levelno >= 20:
            color = '\x1b[32m'  # INFO green
        elif levelno >= 10:
            color = '\x1b[90m' # DEBUG light gray
        else:
            color = '\x1b[0m'  # normal
        try:
            new_args[1].msg = color + ensure_text(args[1].msg) + '\x1b[0m'  # normal
        except Exception as e:
            raise e
        return fn(*new_args)
    return new

logging.StreamHandler.emit = add_coloring_to_emit_ansi(logging.StreamHandler.emit)
